@@ -27,8 +27,8 @@ DNS::Unbound::AsyncQuery
 This object represents the result of an asynchronous L<DNS::Unbound> query.
 It subclasses L<Promise::ES6> but provides a cancellation mechanism.
 
-The promise resolves with the same hash that C<DNS::Unbound::resolve()>
-returns. It rejects with a L<DNS::Unbound::X::ResolveError> instance
+The promise resolves with a L<DNS::Unbound::Result> instance.
+It rejects with a L<DNS::Unbound::X::ResolveError> instance
 that describes the failure.
 
 =cut
@@ -51,6 +51,29 @@ class provides:
 
 =cut
 
+=head2 I<OBJ>->cancel()
+
+Cancels an in-progress DNS query. Returns nothing.
+
+B<NOTE:> This will leave the promise I<unresolved>.
+
+=cut
+
+sub cancel {
+    my ($self) = @_;
+
+    my $dns_hr = $self->_get_dns();
+
+    if (!$dns_hr->{'fulfilled'}) {
+        if (my $ctx = delete $dns_hr->{'ctx'}) {
+            delete $dns_hr->{'queries_lookup'}{ $dns_hr->{'id'} };
+            $CANCEL_CR->( $ctx, $dns_hr->{'id'} );
+        }
+    }
+
+    return;
+}
+
 # Leaving undocumented since as far as the caller is concerned
 # this method is identical to the parent class’s.
 sub then {
@@ -63,43 +86,30 @@ sub then {
     return $new;
 }
 
-=head2 I<OBJ>->cancel()
-
-Cancels an in-progress DNS query. Returns nothing.
-
-=cut
-
-sub cancel {
-    my ($self) = @_;
-
-    my $dns_hr = $self->_get_dns();
-
-    if (!$dns_hr->{'fulfilled'}) {
-        if (my $ctx = delete $dns_hr->{'ctx'}) {
-            $CANCEL_CR->( $ctx, $dns_hr->{'id'} );
-        }
-    }
-
-    return;
-}
-
 # ----------------------------------------------------------------------
 # Interfaces for DNS::Unbound to interact with the query’s DNS state.
 # Nothing external should call these other than DNS::Unbound.
 
+my %QUERY_OBJ_DNS;
+
 sub _set_dns {
     my ($self, $dns_hr) = @_;
-    $self->{'_dns'} = $dns_hr;
+
+    $QUERY_OBJ_DNS{$self} = $dns_hr;
+
     return $self;
 }
 
 sub _get_dns {
-    return $_[0]->{'_dns'};
+    return $QUERY_OBJ_DNS{$_[0]};
 }
 
-sub _forget_dns {
-    delete $_[0]->{'_dns'};
-    return $_[0];
+sub DESTROY {
+    my $self = shift;
+
+    delete $QUERY_OBJ_DNS{$self};
+
+    return;
 }
 
 1;
